function [Y,Ymask,Ytrue,posc] = create_Y(MD, matData, all_place, signal_length, signal_shift)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Input:
%  MD               = data (missing values, detrend)
%  matData          = data without missing values
%  all_place        = what time series are used (cols of MD)
%  signal_length    =  length of a measurements vector (number of rows in Y)
%  signal_shift     = shift from one signal to the next
% Output:
%  Y                = matrix of training signals (with 0 instead of missing measurements);
%                     number of signals = (length_data-signal_length) / signal_shift * length(all_place)
%  Ymask            = mask of available measurements (1=valid measurement, 0=missing or invalid measurement)
%  Ytrue            = Y without missing data
%  posc             = indexes of the columns of Y where time series begin
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% BD 8.05.2019 Modified CDG 14.10.2019

first_data = 1;                % index of first data (position in the matrix)
length_data = size(MD,1);      % total number of measurements that is used (rows in the matrix)

% extract useful matrix
h       = MD(first_data:first_data+length_data-1, all_place);
htrue   = matData(first_data:first_data+length_data-1, all_place);

% create training signal matrix
N = floor((length_data - signal_length + 1) / signal_shift) * length(all_place);
Y = zeros(signal_length, N);
Ytrue = Y;

c = 1;
posc = zeros(1,length(all_place));
for j = 1 : length(all_place)
  for i = 1 : signal_shift : length_data - signal_length + 1
    if i==1
        posc(j) = c;
    end
    Y(:,c)      = h(i:i+signal_length-1,j);
    Ytrue(:,c)  = htrue(i:i+signal_length-1,j);
    c           = c+1;
  end
end

% mask matrix
Ymask = (Y ~= 0);

end %function
