function [matData,matData_miss] = get_data(fname_c,fname_miss,perc_miss,flag_polya,par_polya)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Input:
% fname_c           = file name complete dataset
% fname_miss        = file name missing data
% perc_miss         = percentage missing data
% flag_polya        = if 1, use Polya urn for positions missing data
%                     if 0, use unif. distrib. for positions missing data 
% par_polya         = vector with two entries:
%                     1st entry: M-"memory" Polya urn
%                     2nd entry: delta-if large, there are bursts of 
%                     missing data instead of isolated missing data
% Output:
% matData           = complete dataset
% matData_miss      = missing data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Complete data
if isfile(fname_c)==0
    fprintf('ERROR: the file %s does not exist!\n', fname_c);
    matData = [];
    matData_miss = [];
    return;
else
    load(fname_c,'matData');
end

%Missing data
flag = 0;
if isfile(fname_miss)==0
    flag = 1;
else
    load(fname_miss,'matData_miss','perc');
    if size(matData)~=size(matData_miss)
        flag = 1;
    else
        temp = matData;
        temp(matData_miss==0) = 0;
        if sum(sum(temp-matData_miss)) ~=0 
            flag = 1;
        end
    end
    if perc~=perc_miss
        flag = 1;
    end
    if flag_polya==1
        load(fname_miss,'M','delta');
        if (M~=par_polya(1)) || (delta~=par_polya(2))
            flag = 1;
            fprintf('M=%i delta=%5.4f\n', M, delta);
        end
    end
end

if flag == 1
    %force missing values
    matData_miss = matData;
    perc = perc_miss;
    if flag_polya==1
        for k = 1 : size(matData_miss,2)
            ind = ind_polya(perc, size(matData_miss,1), par_polya);
            matData_miss(ind,k)=0;
            M = par_polya(1);
            delta = par_polya(2);
            save(fname_miss,'matData_miss','perc','M','delta');    
        end
    else   
        for k = 1 : size(matData_miss,2)
            ind = randperm(size(matData_miss,1), round(perc * size(matData_miss,1)));
            matData_miss(ind,k)=0;
        end
        save(fname_miss,'matData_miss','perc');
    end
end %flag

end %function

