function ind= ind_polya(PERC_MISS, Nts, par_polya)
%Input:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PERC_MISS     = percentage missing data
% Nts           = total no. of observations for the time series
% par_polya     = vector with two entries:
%                     1st entry: M-"memory" Polya urn
%                     2nd entry: delta-if large, there are bursts of 
%                     missing data instead of isolated missing data
%Output:
% ind           = positions missing data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Urn
T = 10^3;
R = floor(PERC_MISS*T);
S = T-R;

M =     par_polya(1);
delta = par_polya(2);
Delta = delta*T;

N   = min(10^2*Nts, 10^7);
err = zeros(N,1);

%Init
for i=1:M
    ball = binornd(1,R/T);
    if ball==1
        err(i)=1;
        R = R+Delta;
    else
        S = S+Delta;
    end
    T = R+S;
end

%Main loop
for i=M+1:N
    ball = binornd(1,R/T);
    if ball==1
        err(i)=1;
        R=R+Delta;
    else
        S=S+Delta;
    end
    if err(i-M)==1
        R = R-Delta;
    else
        S = S-Delta;
    end
    T = S+R; %should be Tinit + M*Delta   
end

%Select a block of length Nts
diff = Inf;
for t=M+1:N-Nts+1
    vtemp = err(t:t+Nts-1);
    if abs( sum(vtemp)/length(vtemp)-PERC_MISS ) < diff
        tt = t;
        diff = abs( sum(vtemp)/length(vtemp)-PERC_MISS );
    end
end

err_out = err(tt:tt+Nts-1);
ind = find(err_out==1);
    
end %function

