function itc = it_crt(Y, Ymask, D, X, no_it_crt)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Input:
%Input:
% Y             = Y used in training 
% Ymask         = Ymask used in training 
% D             = dictionary
% X             = representation
% no_it_crt     = no. of IT criteria that are evaluated
%Output:
% itc           = vector that contains the values of the IT criteria
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Modified Jan. 13,2020

%Dimensions
[m,n] = size(D);
N = size(Y,2);

%No. of measurements
T = sum(sum(Ymask));

%No. of parameters
x = (X(:)~= 0);
nop = sum(x);
s = round(nop/N);
nnop = nop + (m - 1) * n;

%Errors
Yres = D*X;
rmse = norm((Y-Yres).*Ymask,'fro')/sqrt(T);
gof = 2*log(rmse);

%Init criteria
itc = Inf*ones(no_it_crt,1);

%BIC2 (written with nnop)
itc(1) = gof + (log(T)/T) * nnop;

%EBIC2
itc(2) = itc(1) + (2*N/T) * ( gammaln(n+1)-gammaln(s+1)-gammaln(n-s+1) );

end %function

        
        


    
