function [matData_miss_d, XvecAB] = lin_trend_periodic_comp(MS, vflag, vperiod)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Input:
% MS                = column vector with missing data
% vflag             = [flag_log flag_lin_trend flag_periodic_comp flag_season]
% vperiod           = periods for the periodic components (used only if flag_periodic_comp~0)
%Output:
% matData_miss_d    = column vector with missing data, after removing the linear
%                     trend and the sinusoidal components 
% XvecAB            = lin. + periodic components 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

vperiod         = vperiod(:);
vperiod         = vperiod';
matData_miss_d  = zeros(size(MS));

ff  = find(MS~=0);
X   = Inf*ones(length(ff),2*length(vperiod)*vflag(3)+2*vflag(2));
    
if vflag(2)~=0
    X(:,1:2) = [ones(length(ff),1) ff];
end
    
if vflag(3)~=0
    temp = (2*pi)*kron((1./vperiod),ff);
    X(:,3:end) = [cos(temp) sin(temp)];
end
        
vecAB                   = X\MS(ff);
XvecAB_ff               = X*vecAB;
matData_miss_d(ff)      = MS(ff) - XvecAB_ff;
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Lin. and periodic components
    
LL = length(MS);
X = Inf*ones(LL,2*length(vperiod)*vflag(3)+2*vflag(2));
    
if vflag(2)~=0
    X(:,1:2) = [ones(LL,1) (1:LL)'];
end
    
if vflag(3)~=0
    temp = (2*pi)*kron((1./vperiod),(1:LL)');
    X(:,3:end) = [cos(temp) sin(temp)];
end
XvecAB                  = X*vecAB;
    

end %function

