function [Ytr, Yr] = restore_data(length_data, all_place, all_Ytrue, all_Y, D, X, signal_shift, posc)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Input:
% length_data   = length of the original time series
% all_place     = indexes time series
% all_Ytrue     = original data (no missing data)
% all_Y         = Y used in training 
% D             = dictionary
% X             = representation
% signal_shift  = signal shift
% posc          = begining of each time series in Y 
%Output:
% Ytr           = Ytrue (one col. for each time series)
% Yr            = approx. signal (one col. for each time series)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[m,N1] = size(all_Y);    % size training data
if rem(N1,length(all_place))~=0
    fprintf('ERROR: The no. of cols. of all_Y is not a multiple integer of the no. of time series!\n');
    return;
else
    N = N1/length(all_place);
end
    
all_Yres = D*X;         % restored Y

L = N*signal_shift + (m-signal_shift); % should be equal to length_data 
if L~=length_data
    fprintf('ERROR: L=%i length_data=%i \n', L, length_data);
end

Ytr = zeros(length_data,length(all_place));
Yr  = zeros(length_data,length(all_place));

for ind=1:length(all_place)
    
    yres = zeros(L,1);     % the restored signal
    ytrue = zeros(L,1);    % original data
    nav = zeros(L,1);      % number of signals to average at restoration
    
    if ind~=length(all_place)
        Yres    = all_Yres(:,posc(ind):posc(ind+1)-1);
        Ytrue   = all_Ytrue(:,posc(ind):posc(ind+1)-1);
    else
        Yres    = all_Yres(:,posc(ind):end);
        Ytrue   = all_Ytrue(:,posc(ind):end);
    end
    
    j = 1;
    for i = 1 : signal_shift : L-m+1    % slow version now...
        ytrue(i:i+m-1) = Ytrue(:,j);
        yres(i:i+m-1) = yres(i:i+m-1) + Yres(:,j);
        nav(i:i+m-1) = nav(i:i+m-1) + ones(m,1);
        j = j+1;
    end
    
    yres        = yres ./ nav;
    Yr(:,ind)   = yres;
    Ytr(:,ind)  = ytrue;

end %ind

end %function

