function test_missing_data_multi(fname_c, no_it_crt, iters, m, nv, sv, qv, crun, ...
    perc_miss, flag_polya, par_polya, all_place, signal_shift, all_vflag, ...
    all_vperiod, all_vseason, flag_plots, fname_out)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Input:
% fname_c           = name data file
% no_it_crt         = no. of the IT criteria that are evaluated
% iters             = number of DL iterations
% m                 = signal length
% nv                = vector dictionary sizes
% sv                = vector sparsity levels
% qv                = nv./m
% crun              = current run
% perc_miss         = percentage missing data
% flag_polya        = if 1, use Polya urn for positions missing data
%                     if 0, use unif. distrib. for positions missing data 
% par_polya         = vector with two entries:
%                     1st entry: M-"memory" Polya urn
%                     2nd entry: delta-if large, there are bursts of 
%                     missing data instead of isolated missing data
% all_place         = list of times series that are used 
% signal_shift      = how many sampling periods from the beginning of a signal to the next
% all_vflag         = for all time series: [flag_log flag_lin_trend flag_periodic_comp flag_season]
% all_vperiod       = for all time series: periods for the periodic components (used only if flag_periodic_comp~0)
% all_vseason       = for all time series: seasonality parameters (used only if flag_season~=0)
% flag_plots        = if 0, does not show the plots
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if flag_polya==1
    % file with missing data
    fname_miss              = strcat(fname_c,'_miss_',num2str(round(perc_miss*100)),'_polya_M_',num2str(par_polya(1)),'_delta_',num2str(round(100*par_polya(2))),'.mat');    
    
    % file with transformed data
    fname_transf            = strcat(fname_c,'_transf_',num2str(round(perc_miss*100)),'_polya_M_',num2str(par_polya(1)),'_delta_',num2str(round(100*par_polya(2))),'.mat'); 
    % file with the periodic components
    fname_period            = strcat(fname_c,'_periodic_comp_',num2str(round(perc_miss*100)),'_polya_M_',num2str(par_polya(1)),'_delta_',num2str(round(100*par_polya(2))),'.mat');    
    % file with the seasonal components
    fname_season            = strcat(fname_c, '_season_comp_',num2str(round(perc_miss*100)),'_polya_M_',num2str(par_polya(1)),'_delta_',num2str(round(100*par_polya(2))),'.mat');       
else
    fname_miss              = strcat(fname_c,'_miss_',num2str(round(perc_miss*100)),'.mat');                % file with missing data
    fname_transf            = strcat(fname_c,'_transf_',num2str(round(perc_miss*100)),'.mat');              % file with transformed data
    fname_period            = strcat(fname_c,'_periodic_comp_',num2str(round(perc_miss*100)),'.mat');       % file with the periodic components
    fname_season            = strcat(fname_c, '_season_comp_',num2str(round(perc_miss*100)),'.mat');        % file with the seasonal components
end %flag_polya

%Load data
[matData,matData_miss]  = get_data(strcat(fname_c,'.mat'),fname_miss,perc_miss,flag_polya,par_polya);

%Transform data
max_no_s_comp   = 2; %max. no. of seasonal components !!!!!!
[matData_transf, matData_period, matData_season] = transform_data(matData_miss, fname_transf, fname_period, fname_season, all_place, all_vflag, all_vperiod, all_vseason, max_no_s_comp);

%File for saving the results
if isfile(fname_out)
    load(fname_out,'err_cell_miss', 'err_cell_miss_indiv', 'cell_crit');
else
    fprintf('ERROR: fname_out does not exist!\n');
    return;
end
    
%Create training data
[Y,Ymask,Ytrue,Ypos] = create_Y(matData_transf, matData, all_place, m, signal_shift);

D00 = normc(randn(m,length(all_place)*max(nv)));        % initialize the dictionary randomly (sampling signals is difficult due to the missing data)

for a = 1 : length(nv)                                  % for all  dictionary sizes 
    n = nv(a);                                          % current number of atoms is n
   
    for aa = 1:(qv(a)-1)
        d_indi = m*aa;                                  %size individual dictionaries  
        d_size = [ones(1,length(all_place))*d_indi n-d_indi];
        D0 = D00(:,1:sum(d_size));                      % init. dictionary
       
        for b = 1 : length(sv)                          % for all sparsity levels
            s = sv(b);                                  % current sparsity level is s
        
            %Dictionary learning
            [D,X] = DL_mask_multi(Y, Ymask, Ypos, D0, d_size, s, iters);
            
            %Compute IT criteria
            temp                 = cell_crit{a,1};
            temp(:,crun,aa,b)    = it_crt(Y, Ymask, D, X, no_it_crt);
            cell_crit{a,1}       = temp;
            
            %Restore data
            [Ytr, Yr] = restore_data(size(matData_transf,1), all_place, Ytrue, Y, D, X, signal_shift, Ypos);
            if sum(sum(abs( Ytr-matData(:,all_place) )))~=0
                fprintf('ERROR: The data was not correctly used!\n');
                return;
            end
            
            %Add trend
            for ind=1:length(all_place)
                place = all_place(ind);
                vflag = all_vflag{place};
                %seasonality
                if vflag(4)~=0
                    Yr(:,ind) = Yr(:,ind) + sum(matData_season(:,(place-1)*max_no_s_comp+1:place*max_no_s_comp),2);
                end
                %linear trend & periodic components
                if numel( find(vflag(2:3)~=0) )~=0
                    Yr(:,ind) = Yr(:,ind) + matData_period(:,place);
                end
                %log-transformed data
                if vflag(1)~=0
                    Yr(:,ind) = exp(Yr(:,ind));
                end
            end %ind
            
            %Compute errors
            sum_err=0;
            for ind=1:length(all_place)
                ff              = find(matData_miss(:,all_place(ind))==0);
                err_indiv       = norm(Ytr(ff,ind)-Yr(ff,ind))/norm(Ytr(ff,ind));
                if isnan(err_indiv)
                    fprintf('ind=%i NormYr=%5.4f\n', ind, norm(Yr(ff,ind)));
                end
                sum_err                       = sum_err + err_indiv;
                temp                          = err_cell_miss_indiv{a,1};
                temp(ind,crun,aa,b)           = err_indiv;
                err_cell_miss_indiv{a,1}      = temp; 
            end
            temp                = err_cell_miss{a,1};
            temp(crun,aa,b)     = sum_err/length(all_place);
            err_cell_miss{a,1}  = temp; 
            
            %Save results
            save(fname_out,'err_cell_miss', 'err_cell_miss_indiv', 'cell_crit');
            
            %Plots
            if flag_plots==1
                npl = min(3,length(all_place));
                figure(1);
                for ind=1:npl
                    subplot(1,npl,ind);
                    plot(1:size(matData_transf,1),Ytr(:,ind),'r',1:size(matData_transf,1),Yr(:,ind),'k');
                    legend('True','Est');
                end
            end %flag_plots
            
        end %b
    end %aa
end %a

end %function

