function [matData_transf, matData_period, matData_season] = transform_data(matData_miss, fname_transf, fname_period, fname_season, all_place, all_vflag, all_vperiod, all_vseason, max_no_s_comp)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Input:
% matData_miss      = data matrix with missing values
% fname_transf      = name file transformed data
% fname_period      = name file lin. and periodic components
% fname_season      = name file seasonal components
% all_place         = vector with the indexes of the time series considered
%                     for DL
% all_vflag         = for each TS, it is a vector [flag_log flag_lin_trend flag_periodic_comp flag_season]
% all_vperiod       = for each TS, periods for the periodic components (used only if flag_periodic_comp~0)
% all_vseason       = for each TS, seasonality parameters (used only if flag_season~=0)
% max_no_s_comp     = max. no. of seasonal components
%Output:
% matData_transf    = transformed data
% matData_period    = lin. and periodic components
% matData_season    = seasonal components
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Test if the data should be transformed
total           = 0;
flag_lin_period = 0;
flag_season     = 0;
max_len_vseason = 0;
for ind=1:length(all_place)
    place = all_place(ind);
    vflag = all_vflag{place};
    if numel( find(vflag(2:3)~=0) )~=0
        flag_lin_period = 1;
    end
    if vflag(4)~=0
        flag_season = 1;
        max_len_vseason = max(max_len_vseason,length(all_vseason{place}));
        if max_len_vseason>max_no_s_comp
            fprintf('The data are not transformed because the no. of seasonal componets cannot be greater than 2!\n');
            matData_transf = matData_miss;
            matData_period = [];
            matData_season = [];
            return;
        end
    end
    total = total + sum(vflag);
end %for ind
if total==0
    matData_transf = matData_miss;
    matData_period = [];
    matData_season = [];
    return;
end

flag_exist = 1;

%file for the transformed data
if isfile(fname_transf)
    load(fname_transf,'matData_transf');
else
    matData_transf = Inf*ones(size(matData_miss));
    flag_exist = 0;
end

%File for the lin. and periodic components
if flag_lin_period == 1
    if isfile(fname_period)
        load(fname_period,'matData_period');
    else
        matData_period = zeros(size(matData_miss));
        flag_exist = 0;
    end
else
    matData_period = [];
end

%File for the seasonal components
if flag_season == 1
    if isfile(fname_season)
        load(fname_season,'matData_season');
    else
        matData_season  = zeros(size(matData_miss,1),max_no_s_comp*size(matData_miss,2));
        flag_exist = 0;
    end
else
    matData_season = [];
end
    

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %Transform data
 if flag_exist==0
  for place = 1:size(matData_miss,2) %transform all the time series
 
    MS      = matData_miss(:,place); %MS is a column vector!!!!
    vflag   = all_vflag{place};
    vperiod = all_vperiod{place};
    vseason = all_vseason{place};
    
    %log-transformed data
    if vflag(1)~=0
        MS(MS~=0) = log( MS(MS~=0) );
    end
    
    %linear trend & periodic components
    if numel( find(vflag(2:3)~=0) )~=0
        [MS, XvecAB] = lin_trend_periodic_comp(MS, vflag, vperiod);
        matData_period(:,place) = XvecAB;
        save(fname_period,'matData_period');
    end
    
    %seasonality
    if vflag(4)~=0
        [MS, mat_past] = season_comp(MS, vseason);
        matData_season(:,max_no_s_comp*(place-1)+1:max_no_s_comp*(place-1)+length(vseason)) = mat_past;
        save(fname_season,'matData_season');
    end

    matData_transf(:,place) = MS;
    save(fname_transf,'matData_transf'); %save transformed data
    
  end %for place
 end %if flag_exist


end %function

