function [matData_miss_d, matAB, XvecAB] = lin_trend_periodic_comp(matData_miss, vflag, vperiod)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Input:
% matData_miss      = matrix with missing data
% vflag             = [flag_log flag_lin_trend flag_periodic_comp flag_season]
% vperiod           = periods for the periodic components (used only if flag_periodic_comp~0)
%Output:
% matData_miss_d    = matrix with missing data, after removing the linear
%                     trend and the sinusoidal components 
%                     (the trend is computed for each col.)
% matAB             = lin. coeff. each column
% XvecAB            = lin. + periodic components 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

vperiod         = vperiod(:);
vperiod         = vperiod';
matData_miss_d  = zeros(size(matData_miss));
matAB           = zeros(2*length(vperiod)*vflag(3)+2*vflag(2),size(matData_miss,2));

for k=1:size(matData_miss,2)
    
    ff  = find(matData_miss(:,k)~=0);
    X   = Inf*ones(length(ff),2*length(vperiod)*vflag(3)+2*vflag(2));
    
    if vflag(2)~=0
        X(:,1:2) = [ones(length(ff),1) ff];
    end
    
    if vflag(3)~=0
        temp = (2*pi)*kron((1./vperiod),ff);
        X(:,3:end) = [cos(temp) sin(temp)];
    end
        
    vecAB                   = X\matData_miss(ff,k);
    matAB(:,k)              = vecAB;
    XvecAB_ff               = X*vecAB;
    matData_miss_d(ff,k)    = matData_miss(ff,k) - XvecAB_ff;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %Periodic components
    LL = size(matData_miss,1);
    X = Inf*ones(LL,2*length(vperiod)*vflag(3)+2*vflag(2));
    
    if vflag(2)~=0
        X(:,1:2) = [ones(LL,1) (1:LL)'];
    end
    
    if vflag(3)~=0
        temp = (2*pi)*kron((1./vperiod),(1:LL)');
        X(:,3:end) = [cos(temp) sin(temp)];
    end
    XvecAB                  = X*vecAB;
    
%    if k==18
%         vecAB
%         figure(7);
%         plot(1:length(matData_miss(:,k)),matData_miss(:,k),'r',1:length(matData_miss(:,k)),matData_miss_d(:,k));
%         legend('Before','After');
%         figure(8);
%         plot(ff, (X*vecAB));
%         pause
%    end
    
end %k


end %function

