function test_missing_data(fname_c, no_it_crt, iters, m, nv, sv, crun, no_runs, perc_miss, flag_polya, par_polya, place, signal_shift, vflag, vperiod, vseason, flag_plots)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Input:
% fname_c
% no_it_crt         = no. of the IT criteria that are evaluated
% iters             = number of DL iterations
% m                 = signal length, normally a multiple of 12 (we work with monthly data)
% nv                = vector dictionary sizes
% sv                = vector sparsity levels
% crun              = current run
% no_runs           = total number of runs
% perc_miss         = percentage missing data
% flag_polya        = if 1, use Polya urn for positions missing data
%                     if 0, use unif. distrib. for positions missing data 
% par_polya         = vector with two entries:
%                     1st entry: M-"memory" Polya urn
%                     2nd entry: delta-if large, there are bursts of 
%                     missing data instead of isolated missing data
% place             = list of times series that are used 
% signal_shift      = how many months from the beginning of a signal to the next (divisor of 12)
% vflag             = [flag_log flag_lin_trend flag_periodic_comp flag_season]
% vperiod           = periods for the periodic components (used only if flag_periodic_comp~0)
% vseason           = seasonality parameters (used only if flag_season~=0)
% flag_plots        = if 0, does not show the plots
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if flag_polya==1
    % file with missing data
    fname_miss              = strcat(fname_c,'_miss_',num2str(round(perc_miss*100)),'_polya_M_',num2str(par_polya(1)),'_delta_',num2str(round(100*par_polya(2))),'.mat');    
    
    % file with transformed data
    fname_transf            = strcat(fname_c,'_transf_',num2str(round(perc_miss*100)),'_polya_M_',num2str(par_polya(1)),'_delta_',num2str(round(100*par_polya(2))),'.mat'); 
    % file with the periodic components
    fname_period            = strcat(fname_c,'_periodic_comp_',num2str(round(perc_miss*100)),'_polya_M_',num2str(par_polya(1)),'_delta_',num2str(round(100*par_polya(2))),'.mat');    
    % file with the seasonal components
    fname_season            = strcat(fname_c, '_season_comp_',num2str(round(perc_miss*100)),'_polya_M_',num2str(par_polya(1)),'_delta_',num2str(round(100*par_polya(2))),'.mat');       
else
    fname_miss              = strcat(fname_c,'_miss_',num2str(round(perc_miss*100)),'.mat');                % file with missing data
    fname_transf            = strcat(fname_c,'_transf_',num2str(round(perc_miss*100)),'.mat');              % file with transformed data
    fname_period            = strcat(fname_c,'_periodic_comp_',num2str(round(perc_miss*100)),'.mat');       % file with the periodic components
    fname_season            = strcat(fname_c, '_season_comp_',num2str(round(perc_miss*100)),'.mat');        % file with the seasonal components
end %flag_polya

%load data
[matData,matData_miss]  = get_data(strcat(fname_c,'.mat'),fname_miss,perc_miss,flag_polya,par_polya);

%file for the transformed data
if isfile(fname_transf)
    load(fname_transf,'matData_transf');
else
    matData_transf = Inf*ones(size(matData_miss));
end

%File for the lin. and periodic components
if isfile(fname_period)
    load(fname_period,'matData_period');
else
    matData_period = zeros(size(matData_miss));
end

%File for the seasonal components
max_no_s_comp   = 2; %max. no. of seasonal components !!!!!!
if isfile(fname_season)
    load(fname_season,'matData_season');
else
    matData_season  = zeros(size(matData_miss,1),max_no_s_comp*size(matData_miss,2));
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %Transform data
 %Note: the transformed data is re-computed at each run
 %Later on, we should modify the code such that to compute 
 %the transformed data only in the first run
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 MS = matData_miss(:,place); %MS is a column vector!!!!
 %log-transformed data
 if vflag(1)~=0
    MS(MS~=0) = log( MS(MS~=0) );
 end
 %linear trend & periodic components
if numel( find(vflag(2:3)~=0) )~=0
    [MS, ~, XvecAB]     = lin_trend_periodic_comp(MS, vflag, vperiod);
    matData_period(:,place) = XvecAB;
    save(fname_period,'matData_period');
end
%seasonality
if vflag(4)~=0
    [MS, mat_past] = season_comp(MS, vseason);
    matData_season(:,max_no_s_comp*(place-1)+1:max_no_s_comp*(place-1)+length(vseason)) = mat_past;
    save(fname_season,'matData_season');
end

matData_transf(:,place) = MS;
save(fname_transf,'matData_transf'); %save transformed data
    
if crun==1
    %Init
    err_mat_miss    = zeros(no_runs, length(nv), length(sv));                        %errors in the estimation of the missing data
    mat_crit        = zeros(no_it_crt, no_runs, length(nv), length(sv));             %values IT criteria
else
    load('Outputs_Temp.mat','err_mat_miss','mat_crit');
end %crun

%create training data
[Y,Ymask,Ytrue,posc] = create_Y(matData_transf, matData, place, m, signal_shift);

D00 = normc(rand(m,max(nv)));  % initialize the dictionary randomly (sampling signals is difficult due to the missing data)

pl = 1;
for a = 1 : length(nv)      % for all  dictionary sizes 
    n = nv(a);              % current number of atoms is n
    D0 = D00(:,1:n);        % init. dictionary
    for b = 1 : length(sv)      % for all sparsity levels
        s = sv(b);              % current sparsity level is s
        
        %dictionary learning
        [D, X, ~, ~] = ...
        DL(Y, D0, s, iters, str2func('aksvd_mask'), 'replatoms', 'random', ...
        'spfunc', str2func('omp_mask'), 'data_mask', Ymask, 'outopts', 'last');  
      
        %evaluate IT criteria
         mat_crit(:,crun,a,b) = it_crt(Y, Ymask, D, X, no_it_crt);
        
        %restore data
        [ytrue, ~, yres, ~, ~] = restore_data(matData, place, Ytrue, Y, D, X, signal_shift, posc);
        
        %add trend
        yr = yres;
        %seasonality
        if vflag(4)~=0
            yr = yr + sum(mat_past,2);
        end
        %linear trend & periodic components
        if numel( find(vflag(2:3)~=0) )~=0
            yr = yr + XvecAB;
        end
        %log-transformed data
        if vflag(1)~=0
            yr = exp(yr);
        end
        yres_d = yr;
        
        %compute errors
         % ff = find(ymask==0);
         ff                         = find(matData_miss(:,place)==0);
         err_mat_miss(crun,a,b)     = norm(ytrue(ff)-yres_d(ff))/norm(ytrue(ff));
      
        if flag_plots
            
            figure(1)
            subplot(length(sv),length(nv),pl);
            plot(1:length(ytrue),ytrue,'r',1:length(ytrue),yres_d,'k');
            if pl==1
                title('All Data');
                legend('True','Est');
            end 
            
            figure(2)
            subplot(length(sv),length(nv),pl);
            plot(1:length(ytrue),ytrue-yres_d,'b');
            if pl==1
                title('All Data');
                legend('Errors');
            end 
            
            figure(3)
            subplot(length(sv),length(nv),pl);
            plot(1:length(ff),ytrue(ff),'r',1:length(ff),yres_d(ff),'k');
            if pl==1
                title('Missing Data');
                legend('True','Est');
            end 
            
            pl=pl+1;
            
        end %flag_plots
 
    end %b
end %a

save('Outputs_Temp.mat', 'err_mat_miss','mat_crit');    

end %function

