function run_testing_missing_data(no_runs, perc_miss, place, signal_shift, vflag, vperiod, vseason, flag_plots)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Input:

% no_runs           = no. of initializations for the dictionary 
% perc_miss         = percentage missing data
% place             = list of times series that are used 
% signal_shift      = how many months from the beginning of a signal to the next (divisor of 12)
% vflag             = [flag_log flag_lin_trend flag_periodic_comp flag_season]
% vperiod           = periods for the periodic components (used only if flag_periodic_comp~0)
% vseason           = seasonality parameters (used only if flag_season~=0)
% flag_plots        = if 0, does not show the plots
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

close all

if (perc_miss<=0) || (perc_miss>=50/100)
    fprintf('ERROR: perc_miss should be selected from the interval (0,1/2)!\n');
    return;
end

if size(place,2)~=1
    fprintf('This code can be used only for univariate time series!\n');
    return;
end

if (vflag(3)~=0) && (numel(vperiod)==0)
    fprintf('Provide periods for the periodic components!\n');
    return;
end

if (vflag(4)~=0) && (numel(vseason)==0)
    fprintf('Provide parameters for the seasonality!\n');
    return;
end

% name of the data file (without .mat)
fname_c = 'M4_Hourly_Select_20_ts'; 

%Sizes and parameters
no_it_crt = 6;                  % no. of the IT criteria that are evaluated
iters = 50;                     % number of DL iterations
m = 24*1;                       % signal length, normally a multiple of 24 (we work with Hourly data)
nv = 24*(2:8);                  % dictionary sizes(choosen to be the multiplication of signal_length) 
sv = [2 3 4 6];                 % sparsity levels

%Run test
fprintf('\n');
for crun=1:no_runs
    % fprintf('.');
    test_missing_data(fname_c, no_it_crt, iters, m, nv, sv, crun, no_runs, perc_miss, place, signal_shift, vflag, vperiod, vseason, flag_plots);
end
fprintf('\n');

%Read the results
if isfile('Outputs_Temp.mat')==0
    fprintf('ERROR: the file Outputs_Temp.mat does not exist!\n');
    return;
end

load('Outputs_Temp.mat','err_mat_miss','mat_crit');
!del Outputs_Temp.mat
    
% best_n = zeros(no_it_crt+1,1);
% best_s = zeros(no_it_crt+1,1);
% for cc=1:no_it_crt
%     temp = squeeze(mat_crit(cc,:,:));
%     [~,ind] = min(temp(:));
%     [a,b] = ind2sub(size(temp),ind);
%     best_n(cc) = nv(a);
%     best_s(cc) = sv(b);
% end
% %Oracle
% [~,ind] = min(err_mat_miss(:));
% [a,b] = ind2sub(size(err_mat_miss),ind);
% best_n(end) = nv(a);
% best_s(end) = sv(b);

%Save the results
fname_out    = strcat('Outputs_perc_', num2str(round(perc_miss*100)), '_place_');    % file in which the results are saved
for k=1:length(place)
    fname_out = strcat(fname_out,num2str(place(k)),'_');
end
fname_out = strcat(fname_out,'shift_',num2str(signal_shift));
if vflag(1)~=0
    fname_out = strcat(fname_out,'_logdata');
end
if vflag(2)~=0
    fname_out = strcat(fname_out,'_linear');
end
if vflag(3)~=0
    fname_out = strcat(fname_out,'_periodic');
end
if vflag(4)~=0
    fname_out = strcat(fname_out,'_season');
end
fname_out = strcat(fname_out,'.mat');

save(fname_out, 'no_runs', 'err_mat_miss','mat_crit');  


end % function

