function [W, D] = clas_discrim_dl(Y, H, n, s, alpha)
% Design discriminative DL classifier (Section 8.5.2)
% Input:
%   Y     - signal matrix
%   H     - label matrix
%   n     - number of atoms
%   s     - sparsity level
%   alpha - trade-off parameter in the objective
% Output:
%   W     - classifier matrix
%   D     - dictionary for sparse representation

% BD 23.12.2017

Ye = [Y; sqrt(alpha)*H];    % extended signal matrix
iters = 50;                 % number of iterations in DL algorithm
update = 'aksvd';           % dictionary update method
replatom = 'random';        % atom replacement strategy
params = {};
[m,N] = size(Y);            % m=signal length, N=number of training signals
c = size(H,1);              % number of classes
me = m+c;

% Discriminative DL
D0 = randn(me,n);           % initial dictionary
[De, Xe, errs] = DL(Ye, D0, s, iters, str2func(update), params, 'replatom', replatom);

% Extract dictionary and classifier matrix
D = De(1:m,:);
normD = sum(D.*D);
D = normc(D);
W = De(m+1:me,:) / sqrt(alpha);
W = W ./ repmat(normD, c, 1);
