function D = clas_src_dl(Y, H, nv, s)
% Design SRC classifier with dictionaries trained for each class (Section 8.3)
% Input:
%   Y     - signal matrix
%   H     - label matrix
%   nv    - number of atoms in each class dictionary (vector)
%   s     - sparsity level
% Output:
%   D     - concatenated dictionary for sparse representation

% BD 24.12.2017

[m,N] = size(Y);            % m=signal length, N=number of training signals
c = size(H,1);              % number of classes
Nv = sum(H,1);              % number of signals in each class

iters = 50;                 % number of iterations in DL algorithm
update = 'aksvd';           % dictionary update method
replatom = 'random';        % atom replacement strategy
params = {};

% Train a dictionary for each class
D = [];
for i = 1:c
  is = find(H(i,:));        % indices of signals for the current class
  D0 = randn(m,nv(i));      % initial dictionary
  D0 = normc(D0);
  [Di, X, errs] = DL(Y(:,is), D0, s, iters, str2func(update), params, 'replatom', replatom);
  D = [D Di];
end
