function A = clas_src_dl_ker(Y, H, nv, s, kerfun)
% SRC classifier with dictionaries trained for each class, using kernel AK-SVD
% Input:
%   Y        - signal matrix
%   H        - label matrix
%   nv       - number of atoms in each class dictionary (vector)
%   s        - sparsity level
%   kerfun   - handle of kernel function
% Output:
%   A     - concatenated dictionary for kernel sparse representation

% BD 25.12.2017

[m,N] = size(Y);            % m=signal length, N=number of training signals
c = size(H,1);              % number of classes

iters = 50;                 % number of iterations in DL algorithm

% Train a dictionary for each class
A = [];
for i = 1:c
  is = find(H(i,:));        % indices of signals for the current class
  A0 = randn(length(is),nv(i));      % initial dictionary
  A0 = normc(A0);           % cheap, but incorrect normalization; seems to work well
  [Ai, X, errs] = aksvd_ker(Y(:,is), A0, s, kerfun, iters);
  A = [A Ai];
end
