%%% Copyright (c) 2016, Paul Irofti <paul@irofti.net>
%%% All rights reserved.

function [D,X] = atom_up_mask(Y,Ymask,D,X,replatoms,customfunc)
%% Sequential dictionary update loop
% INPUTS:
%   Y          -- training signals set
%   Y_mask     -- signal mask
%   D          -- current dictionary
%   X          -- sparse representations
%   replatoms  -- unused dictionary atoms replacement strategy
%   customfunc -- algorithm specific atom update routine
%                 The function has to respect the API: 
%                   [d,x] = func(Y,D,X,d,x)
%
% OUTPUTS:
%   D -- updated dictionary
%   X -- updated representations

    for j = 1:size(D,2)     
        [~, data_indices, x] = find(X(j,:));
        d = D(:,j);
        D(:,j) = 0;
        if (isempty(data_indices))
            D(:,j) = new_atom(replatoms,Y,D,X,j);
            continue;
        end
        smallX = X(:,data_indices);
        smallY = Y(:,data_indices);
        smallYmask = Ymask(:,data_indices);
        [D(:,j),X(j,data_indices)] = customfunc(smallY,D,smallX,x,smallYmask);
    end
end