% APrU (version OS) vs AK-SVD
% Generates values similar to those from Table 5.1,
% but the values of s and lambda must be changed manually

%-------------------------------------------------------------------------
m = 64;            % problem dimension
n = 128;
N = 8192;          % number of training signals
s = 10;            % sparsity level for AK-SVD
lambda = 0.00024;  % trade-off parameter for 1-norm penalty
iters = 50;        % number of DL iterations

update = 'aksvd';    % dictionary update method
replatom = 'random'; % atom replacement strategy

type = 'sliding';        % patches can be superposed
imdir = [pwd '\img\'];   % image directory
images = {'lena512.bmp', 'barb.bmp', 'peppers.bmp'};   % images
%%-------------------------------------------------------------------------

% Generate data (ONLY ONCE if multiple values of lam are used!!!)
[D0, Y] = gen_img_data(m, n, N, type, imdir, images);

% AK-SVD
params = {};
[D1, X1, errs1] = DL(Y, D0, s, iters, str2func(update), params, 'replatom', replatom);
fprintf('AK-SVD RMSE: %f\n', min(errs1))

% APrU
[D2, X2, errs2] = apru(Y, D0, lambda, iters, s);
fprintf('APrU RMSE:   %f\n', min(errs2))
fprintf('APrU average sparsity level %.2f\n', sum(sum(X2~=0))/N)

