function [X, shared] = omp_mask(Y, D, s, shared, varargin)

% Orthogonal Matching Pursuit algorithm for data with missing entries.
% The missing entries are simply ignored (masked). 
% Input:
%   Y     - training signals set
%   Ymask - varargin{1} = signal mask
%   D     - current dictionary
%   s     - sparsity level
%
% Output:
%   X     - sparse representations

% BD 10.04.2018

[~,N] = size(Y);
n = size(D,2);    % BD 13.05.2019
X = zeros(n,N);

Ymask = varargin{1};
Y = Y .* Ymask;    % apply mask, to be sure

ompparams = {'checkdict', 'off'};

for i = 1:N    % represent signals one by one, since the available entries are different
  suppy = find(Ymask(:,i));
  if ~isempty(suppy)     % BD 13.05.2019
    Dsmall = D(suppy,:);
    normDsmall = sqrt( sum( Dsmall.*Dsmall ) );
    Dsmall = Dsmall ./ repmat(normDsmall, size(Dsmall,1), 1);
    X(:,i) = omp_sparse(Dsmall'*Y(suppy,i), Dsmall'*Dsmall, s, ompparams{:});
    X(:,i) = X(:,i) ./ normDsmall';
  end
end
